<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response;
use Teknisa\Model\Util\NovoCodigo;


class ChamadoController extends TeknisaCrud
{
    protected $dataSourceName = 'getChamado';
    
    protected $entityManager;
    protected $connection;
    protected $newCode;
    protected $environment;
    protected $casoService;
    protected $enviarMensagemController;
    protected $parameterBagColumns = array('P_NRORG');

    public function __construct ($crudService, $parameterBag, $entityManager, $newCode, $environment, $chamadoService, $enviarMensagemController, $tagController, $ticketBotService, $pessoaRelacionadaController)
    {
        $this->entityManager = $entityManager;
        $this->newCode       = $newCode;
        $this->connection    = $entityManager->getConnection();
        $this->environment   = $environment;
        $this->chamadoService   = $chamadoService;
        $this->enviarMensagemController   = $enviarMensagemController;
        $this->tagController   = $tagController;
        $this->ticketBotService   = $ticketBotService;
        $this->pessoaRelacionadaController   = $pessoaRelacionadaController;

        parent::__construct($crudService, $parameterBag);
    }
    
    public function find(DTO\Request\Filter $request,DTO\Response $response) {
        $this->dataSourceName = "getChamado";
        $hasTagFilter = false;
        $hasPersonFilter = false;
        
        foreach($request->getFilterCriteria()->getConditions() as $filter){
            if($filter['columnName'] === 'IDNOMETAG')
                $hasTagFilter = true;
            if($filter['columnName'] === 'NRPARCNEGOCIORELAC')
                $hasPersonFilter = true;
        }
        
        if(!$hasTagFilter)
            $request->getFilterCriteria()->addCondition('IDNOMETAG', '=', '');
        if(!$hasPersonFilter)
            $request->getFilterCriteria()->addCondition('NRPARCNEGOCIORELAC', '=', '');
        
        $this->parameterBagColumns = array('P_NRORG', 'IDNOMETAG', 'NRPARCNEGOCIORELAC');
        parent::find($request, $response);    
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $externo;
        foreach($request->getDataSet()->getRows() as &$row) {
            
            if($row['__is_new']){
                //Campos obrigatório
                $row['CDFILIAL'] = $this->environment->getCdFilial();
                $row['NRORG'] = $this->environment->getNrOrgTrab();
                $row['NRSEQCONTATO'] = $this->newCode->getCodeByTableName("NRSEQCONTATO", $row["NRORG"]);
                $row['NRCONTATO'] = $row['NRSEQCONTATO'];
                $row['IDTIPOCONTATO'] = 'CHAMADO';
                $row['DSARVOREPROD'] = $row['NMPRODUTO'];
                $row['IDNOVOTICKET'] = 'S';
                $row['DTHRCONCCONT'] = '';
                $row['IDCANCELADO'] = 'N';
                $row['NRSEQSITUACAO'] = empty($row['NRSEQSITUACAO']) ? 1 : $row['NRSEQSITUACAO'];
                
                if(empty($row['CDOPERCONT']) && !isset($row['CHAMADOEXTERNO']))
                    $row['CDOPERCONT'] = $this->environment->getCdoperador();
                    
                //Tratando CDCASO
                if(empty($row['CDCASO']))
                    $row['CDCASO'] = (int)$row['NRSEQCONTATO'];
                
                //Se não for externo e o operador responsável não for o logado, notifica o responsável
                if($row['CDOPERCONT'] !== $this->environment->getCdoperador() && !isset($row['CHAMADOEXTERNO'])){
                    $this->notificaResponsavel($row);
                }
                
                $this->callSendMailOpenedTicket($row);
                $response->addDataSet(new DataSet("DADOS", array("NRSEQCONTATO" => $row['NRSEQCONTATO'])));
            }else{
                if(!isset($row['CHAMADOEXTERNO']))
                    $row['IDNOVOTICKET'] = 'N';
            }
            $externo = isset($row['CHAMADOEXTERNO']) ? true : false;
        }
        $row['IDNOMETAG']           = isset($row['IDNOMETAG']) ? $row['IDNOMETAG'] : '';
        $row['NRPARCNEGOCIORELAC']  = isset($row['NRPARCNEGOCIORELAC']) ? $row['NRPARCNEGOCIORELAC'] : '';
        
        parent::save($request, $response);
        
        $this->tagController->saveOrUpdateTags($row['NRSEQCONTATO'], $row['IDNOMETAG'], 'CONTATO');
        
        if(!$externo){
            $this->pessoaRelacionadaController->saveOrUpdatePessoas($row['NRSEQCONTATO'], $row['NRPARCNEGOCIORELAC']);
        }
    }
    
    public function transfereOperador(DTO\Request $request, Response $response) {
        
        $error = array();
        foreach($request->getDataSet()->getRows() as $row){
        
            $retorno = $this->chamadoService->transfereOperador($row);
            if($retorno != $row["NRSEQCONTATO"]){
                $error[] =  $retorno;
            }
        }
        
        if(!empty($error)){
            $response->setError(new Error( array("ERRO" => $error, "MESSAGE" => 'Não foi possível transferir o chamado.'), 10));
        }else{
            $response->addDataSet(new DataSet("DADOS", array("NRSEQFILACONTA" => $retorno)));
        }
    }
    
    public function saveChamadoExterno(DTO\Request $request, DTO\Response $response){
        foreach($request->getDataSet()->getRows() as &$row) {
            
            $row['CHAMADOEXTERNO']  = true;
            if($row['__is_new']){
                $row['IDSTATUSCONT'] = 'A';
                $row['NRPARCNEGOCIO'] = $this->chamadoService->getNrparcnegocioCliente($this->environment->getCdoperador(), $this->environment->getNrOrg());
                $row['DTHRCONTATO'] = $this->environment->getCurrentTime()->format('d.m.Y H:i:s');
                $row['NRPARCNEGOCIORELAC'] = '';
                $row['IDTPCONTATO'] = 'P';
                $row['CDOPERCONT'] = '';
                $row['IDGERAFATURA'] = 'NAO';
                $row['IDCANCELADO'] = 'N';
                $row['NRSEQSITUACAO'] = 1;
                //Padrão prioridade baixa
                $row['NRPRIORIDADE'] = 4;
                $row['NRCLASSIFICACAOCASO'] = empty($row['NRCLASSIFICACAOCASO']) ? '' : $row['NRCLASSIFICACAOCASO'];
                $row['CDNATUCONT']          = empty($row['CDNATUCONT']) ? '' : $row['CDNATUCONT'];
                $row['DSLOCALCOMPRA'] = '';
                $row['DSARVOREPROD'] = $row['NMPRODUTO'];
                $row['IDNOVOTICKET'] = 'S';
                
                if(empty($row['CDCASO']))
                    $row['CDCASO'] = (int)$row['NRSEQCONTATO'];
                    
                $row['CDMOTICONT'] = empty($row['CDMOTICONT']) ? '' : $row['CDMOTICONT'];
                //todo adicionar email e telefone
                if(!empty($row['EMAIL'])){
                    $comunicaparc = $this->chamadoService->getComunicaparc($row['EMAIL'], array('05'), $row['NRPARCINTERLOCUTOR'], $this->environment->getNrOrg());
                    if(empty($comunicaparc)){
                        $params = array(
                                "CDCOMUNICAPARC" => $row['EMAIL'],
                                "IDATIVO"        => 'S',
                                "NRCOMUNICAPARC" => NovoCodigo::geraCodigo($this->environment->getNrOrg(), "COMUNICAPARC", 1, 10, false, true),
                                "NRPARCNEGOCIO"  => $row['NRPARCINTERLOCUTOR'],
                                "CDFORMACOMU"    => '05',
                                "NRORG"          => $this->environment->getNrOrg()
                            );  
                        if(!$this->chamadoService->insertComunicaparc($params)){
                            $response->addDataSet(new DataSet("ERRO", array("MESSAGE" => 'Não foi possível salvar o email.'), 10));   
                        }
                        
                    }
                }
                if(!empty($row['TELEFONE'])){
                    $comunicaparc = $this->chamadoService->getComunicaparc($row['TELEFONE'], array('01', '02', '03', '04'), $row['NRPARCINTERLOCUTOR'], $this->environment->getNrOrg());
                    if(empty($comunicaparc)){
                        $params = array(
                                "CDCOMUNICAPARC" => $row['TELEFONE'],
                                "IDATIVO"        => 'S',
                                "NRCOMUNICAPARC" => NovoCodigo::geraCodigo($this->environment->getNrOrg(), "COMUNICAPARC", 1, 10, false, true),
                                "NRPARCNEGOCIO"  => $row['NRPARCINTERLOCUTOR'],
                                "CDFORMACOMU"    => '01',
                                "NRORG"          => $this->environment->getNrOrg()
                            );  
                        if(!$this->chamadoService->insertComunicaparc($params)){
                            $response->addDataSet(new DataSet("ERRO", array("MESSAGE" => 'Não foi possível salvar o telefone.'), 10));   
                        }
                    }
                }
            }
        }
        $this->save($request, $response);
    }
    
    public function findChamadoExterno(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = "getChamadoExterno";
        $hasTagFilter = false;
        
        foreach($request->getFilterCriteria()->getConditions() as $filter){
            if($filter['columnName'] === 'IDNOMETAG')
                $hasTagFilter = true;
        }
        if(!$hasTagFilter)
            $request->getFilterCriteria()->addCondition('IDNOMETAG', '=', '');
        
        $this->parameterBagColumns = array('P_NRORG','P_INTERLOC', 'P_PARCTEK', 'IDNOMETAG');
        parent::find($request, $response);
    }
    
    public function notificaInterlocutor(Row $row, Response $response){
        
        $from = $this->environment->getCdOperador();
        $row = $row->getRow();
        
        $interlocutor = $row['NRPARCINTERLOCUTOR'];
        
        $mensagem = array();
        $mensagem['DSTITULOMENSAGEM'] = 'CRM - Notificação de Conclusão de Ticket';
        $mensagem['DSMENSAGEM']    = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:35px !important; font-family: Helvetica, sans-serif; line-height: 24px; ">NOTIFICAÇÃO DE TICKET</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="line-height:22px;padding:20px;"> Prezado(a) [INTERLOCUTOR]</div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px; margin-top:-5px; margin-bottom:15px;"> <p>O Chamado [NRSEQCONTATO] do Produto [PRODUTO] foi encerrado em [DATACONCLUSAO]. </p> <p style="padding-top:15px;margin-bottom:-12px !important;"> <b>Dados do Chamado</b> </p> <ul> <li> <p>Cliente: [CLIENTE] </p> </li> <li> <p>Referência: [REFERENCIA] </p> </li> <li> <p>Operador Responsável: [RESPONSAVEL] </p> </li> </ul> <p> <b>Descrição da Conclusao</b> </p> <p>[DESCRICAO]</p> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table>';
        $mensagem['INTERLOCUTOR']  = $row['INTERLOCUTOR'];
        $mensagem['NRSEQCONTATO']  = $row['NRSEQCONTATO'];
        $mensagem['PRODUTO']       = $row['PRODUTO'];
        $mensagem['DATACONCLUSAO'] = $row['DATACONCLUSAO'];
        $mensagem['CLIENTE']       = $row['CLIENTE'];
        $mensagem['REFERENCIA']    = $row['REFERENCIA'];
        $mensagem['RESPONSAVEL']   = $row['RESPONSAVEL'];
        $mensagem['DESCRICAO']     = $row['DESCCONCLUSAO'];
       
        $this->enviarMensagemController->enviarEmail($mensagem, $row['NRORG'], $interlocutor, $from, $row['EMAIL'], $row['INTERLOCUTOR']);
        
    }
    
    public function createInterloc(Row $row, Response $response){
        try {
            $row = $row->getRow();
            
            $interlocInfo = array(
                    "FROMMAIL" => $row['EMAILINTERLOC'],
                    "FROMNAME" => $row['NMINTERLOCUTOR'],
                    "NRPARCNEGOCIOPARENT" => $row['NRPARCNEGOCIO'],
                    "FROMCPF" => $row['NRCPFINTERLOC']
                );   
            $interlocutor = $this->ticketBotService->createInterloc($interlocInfo);
            
            $response->addDataSet(new DataSet("INTERLOCUTOR", is_array($interlocutor) ? $interlocutor : array($interlocutor)));
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }     
    }
    
    public function callSendMailOpenedTicket($row){
        
        $ticketInfo = array(
                "FROMMAIL" => $row['EMAIL'],
                "FROMNAME" => $row['NMPARCINTERLOCUTOR'],
                "SUBJECT"  => $row['DSREFECONT'],
                "NRPARCINTERLOCUTOR" => $row['NRPARCINTERLOCUTOR']
            ); 
            
        $this->ticketBotService->prepareSendMailOpenedTicket($ticketInfo, $row['NRSEQCONTATO']);
    }
    
    public function notificaResponsavel($row){
        
        $from = $this->environment->getCdOperador();
        
        $responsavel = $row['CDOPERCONT'];
        /*
            //Tira a árvore e deixa só o produto
            $produto = explode("-- ",  $row['NMPRODUTO']);
            $produto = $produto[count($produto)-1];
        */
        
        $mensagem = array();
        $mensagem['DSTITULOMENSAGEM'] = 'CRM - Notificação de Abertura de Ticket';
        $mensagem['DSMENSAGEM']    = '<table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee" style="background-color: #eeeeee;"> <tbody> <tr> <td> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="0274BA" class="container590" style="background-color: #0274ba; text-align: center;"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="center" valign="middle"> <table border="0" align="middle" cellpadding="0" cellspacing="0" style="border-collapse:collapse; mso-table-lspace:0pt; mso-table-rspace:0pt;" class=""> <tbody> <tr> <td align="center" valign="middle"> <p style="color: #ffffff; font-size: 25px; padding:35px !important; font-family: Helvetica, sans-serif; line-height: 24px; ">NOTIFICAÇÃO DE TICKET</p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> <p style="color: #ffffff; font-size: 16px; font-family: " Questrial ", Helvetica, sans-serif; line-height: 15px; padding-top:15px;"></p> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table> <table border="0" width="100%" cellpadding="0" cellspacing="0" bgcolor="eeeeee"> <tbody> <tr> <td align="center"> <table border="0" align="center" width="590" cellpadding="0" cellspacing="0" bgcolor="ffffff" class="container590 bg_color"> <tbody> <tr> <td> <table border="0" align="center" width="540" cellpadding="0" cellspacing="0" class="container580"> <tbody> <tr> <td align="left" style="color: #39434b; font-size: 20px; font-family: " Montserrat ", Helvetica, sans-serif; font-weight: 400; padding-bottom:15px; padding-top:20px; line-height: 35px;" class="title_color"> <div style="line-height:22px;padding:20px;"> Prezado(a) </div> </td> </tr> <tr> <td align="left" class="title_color" style="color: #848999; font-size: 14px; font-family: " Questrial ", sans-serif; mso-line-height-rule: exactly; line-height: 24px; padding-bottom:15px;"> <div style="line-height: 24px; margin-top:-5px; margin-bottom:15px;"> <p>O Ticket [NRSEQCONTATO] do Produto [PRODUTO] foi criado em [DATAINCLUSAO]. </p> <p style="padding-top:15px;margin-bottom:-12px !important;"> <b>Dados do Ticket</b> </p> <ul> <li> <p>Cliente: [CLIENTE] </p> </li> <li> <p>Referência: [REFERENCIA] </p> </li> <li> <p>Operador Responsável: [RESPONSAVEL] </p> </li> </ul> <p> <b>Descrição do Ticket</b> </p> <p>[DESCRICAO]</p> </div> </td> </tr> <tr> <td height="10" style="font-size: 10px; line-height: 10px;">&nbsp;</td> </tr> </tbody> </table> </td> </tr> </tbody> </table> </td> </tr> </tbody> </table>';
        $mensagem['RESPONSAVEL']   = $row['NMOPERCONT'];
        $mensagem['NRSEQCONTATO']  = $row['NRSEQCONTATO'];
        $mensagem['PRODUTO']       = $row['NMPRODUTO'];
        $mensagem['DATAINCLUSAO']  = $this->environment->getCurrentTime()->format('d/m/Y H:i:s');
        $mensagem['CLIENTE']       = $row['NMPARCNEGOCIO']; 
        $mensagem['REFERENCIA']    = $row['DSREFECONT'];
        $mensagem['DESCRICAO']     = isset($row['DSABERTURACASO']) ? $row['DSABERTURACASO'] : '';
        
        $this->enviarMensagemController->enviarEmail($mensagem, $row['NRORG'], $responsavel, null, null, null, true);
        
    }
    
    public function cancelTicket(DTO\Request $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as &$row) {
            $row['IDCANCELADO'] = 'S';
        }
        $this->save($request, $response);
    }
    
}